@@ -3,20 +3,36 @@ seg.Ar.fit.boot<-function(obj, XREG, Z, PSI, opz, n.boot=10, size.boot=NULL, jt=
 #random se TRUE prende valori random quando e' errore: comunque devi modificare qualcosa (magari con it.max)
 #     per fare restituire la dev in corrispondenza del punto psi-random
 #nonParm. se TRUE implemneta il case resampling. Quello semiparam dipende dal non-errore di
-extract.psi<-function(lista){
-#serve per estrarre il miglior psi..
-  dev.values<-lista[[1]][-1] #remove the 1st one referring to model without psi
-  psi.values<-lista[[2]][-1] #remove the 1st one (NA)
-  dev.ok<-min(dev.values)
-    	id.dev.ok<-which.min(dev.values)
-    	if(is.list(psi.values))  psi.values<-matrix(unlist(psi.values),
-    		nrow=length(dev.values), byrow=TRUE)
-    	if(!is.matrix(psi.values)) psi.values<-matrix(psi.values)
-    	psi.ok<-psi.values[id.dev.ok,]
-    	r<-list(SumSquares.no.gap=dev.ok, psi=psi.ok)
-    	r
-	}
+      extract.psi<-function(lista){
+      #serve per estrarre il miglior psi..
+        dev.values<-lista[[1]][-1] #remove the 1st one referring to model without psi
+        psi.values<-lista[[2]][-1] #remove the 1st one (NA)
+        dev.ok<-min(dev.values)
+          	id.dev.ok<-which.min(dev.values)
+          	if(is.list(psi.values))  psi.values<-matrix(unlist(psi.values),
+          		nrow=length(dev.values), byrow=TRUE)
+          	if(!is.matrix(psi.values)) psi.values<-matrix(psi.values)
+          	psi.ok<-psi.values[id.dev.ok,]
+          	r<-list(SumSquares.no.gap=dev.ok, psi=psi.ok)
+          	r
+      	}
 #-------------
+      if(is.null(opz$seed)){
+        mY <- mean(obj$residuals)
+        vv <- strsplit(paste(strsplit(paste(mY),"\\.")[[1]], collapse=""),"")[[1]]
+        vv<-vv[vv!="0"]
+        vv=na.omit(vv[1:5])
+        seed <-eval(parse(text=paste(vv, collapse="")))
+        set.seed(seed)
+      } else {
+        if(is.na(opz$seed)) {
+          seed <-eval(parse(text=paste(sample(0:9, size=6), collapse="")))
+          set.seed(seed)
+        } else {
+          seed <-opz$seed
+          set.seed(opz$seed)
+        }
+      }  
       visualBoot<-opz$visualBoot
       opz.boot<-opz
       opz1<-opz
@@ -24,10 +40,13 @@ extract.psi<-function(lista){
       opz.boot$pow=c(1,1) #c(1.1,1.2)
       opz.boot$it.max<-20
       
-      opz1$it.max <-1
+      opz1$it.max <-0
       n<-nrow(Z)
       o0<-try(suppressWarnings(seg.Ar.fit(obj, XREG, Z, PSI, opz)), silent=TRUE)
       rangeZ <- apply(Z, 2, range) #serve sempre
+      alpha <- opz$alpha
+      limZ <- apply(Z, 2, quantile, names = FALSE, probs = c(alpha, 1 - alpha))
+      
       if(!is.list(o0)) {
           o0<- seg.Ar.fit(obj, XREG, Z, PSI, opz, return.all.sol=TRUE)
           o0<-extract.psi(o0)
@@ -41,7 +60,7 @@ extract.psi<-function(lista){
         } else {
           if(!nonParam) stop("the first fit failed and I cannot extract fitted values for the semipar boot")
           if(random) {
-            est.psi00<-est.psi0<-apply(rangeZ,2,function(r)runif(1,r[1],r[2]))
+            est.psi00<-est.psi0<-apply(limZ,2,function(r)runif(1,r[1],r[2]))
             PSI1 <- matrix(rep(est.psi0, rep(nrow(Z), length(est.psi0))), ncol = length(est.psi0))
             o0<-try(suppressWarnings(seg.Ar.fit(obj, Z, PSI1, opz1)), silent=TRUE)
             ss00<-o0$SumSquares.no.gap
@@ -85,14 +104,14 @@ extract.psi<-function(lista){
           if(is.list(o.boot)){
             all.est.psi.boot[k,]<-est.psi.boot<-o.boot$psi
             } else {
-            est.psi.boot<-apply(rangeZ,2,function(r)runif(1,r[1],r[2]))
+            est.psi.boot<-apply(limZ,2,function(r)runif(1,r[1],r[2]))
             }
           PSI <- matrix(rep(est.psi.boot, rep(nrow(Z), length(est.psi.boot))), ncol = length(est.psi.boot))
           #opz$h<-max(opz$h*.9, .2)
           opz$it.max<-opz$it.max+1
           o <- try(suppressWarnings(seg.Ar.fit(obj, XREG, Z.orig, PSI, opz, return.all.sol=TRUE)), silent=TRUE)
           if(!is.list(o) && random){
-              est.psi0<-apply(rangeZ,2,function(r) runif(1,r[1],r[2]))
+              est.psi0<-apply(limZ,2,function(r) runif(1,r[1],r[2]))
               PSI1 <- matrix(rep(est.psi0, rep(nrow(Z), length(est.psi0))), ncol = length(est.psi0))
               o <- try(suppressWarnings(seg.Ar.fit(obj, XREG, Z, PSI1, opz1)), silent=TRUE)
               count.random<-count.random+1
@@ -128,9 +147,11 @@ extract.psi<-function(lista){
       if(is.null(o0$obj)){
           PSI1 <- matrix(rep(est.psi0, rep(nrow(Z), length(est.psi0))), ncol = length(est.psi0))
           o0 <- try(suppressWarnings(seg.Ar.fit(obj, XREG, Z, PSI1, opz1)), silent=TRUE)
+          warning("The final fit can be unreliable (possibly mispecified segmented relationship)", call.=FALSE, immediate.=TRUE)
       }
       if(!is.list(o0)) return(0)
       o0$boot.restart<-ris
-      rm(.Random.seed, envir=globalenv())
+      o0$seed<-seed
+      #rm(.Random.seed, envir=globalenv())
       return(o0)
       }
\ No newline at end of file
