@@ -8,7 +8,7 @@ seg.glm.fit<-function(y,XREG,Z,PSI,w,offs,opz,return.all.sol=FALSE){
         psi.ok<- psi*h + psi.old*(1-h)
         PSI <- matrix(rep(psi.ok, rep(n, length(psi.ok))), ncol = length(psi.ok))
         U1 <- (Z - PSI) * (Z > PSI)
-        if (pow[1] != 1) U1 <- U1^pow[1]
+        #if (pow[1] != 1) U1 <- U1^pow[1]
         obj1 <- try(suppressWarnings(glm.fit(x = cbind(X, U1), y = y, offset = offs,
                                     weights = w, family = fam, control = glm.control(maxit = maxit.glm), etastart = eta0)),
                                     silent = TRUE)
@@ -37,8 +37,8 @@ seg.glm.fit<-function(y,XREG,Z,PSI,w,offs,opz,return.all.sol=FALSE){
     #--------------------
     in.psi<-function(LIM, PSI, ret.id=TRUE){
         #check if psi is inside the range
-        a<-PSI[1,]<=LIM[1,]
-        b<-PSI[1,]>=LIM[2,]
+        a<-PSI[1,]<LIM[1,]
+        b<-PSI[1,]>LIM[2,]
         is.ok<- !a & !b #TRUE se psi e' OK
         if(ret.id) return(is.ok)
         isOK<- all(is.ok) && all(!is.na(is.ok))
@@ -77,20 +77,21 @@ seg.glm.fit<-function(y,XREG,Z,PSI,w,offs,opz,return.all.sol=FALSE){
     fam<-opz$fam
     maxit.glm<-opz$maxit.glm
     #--------------
-    n<-length(y)
+    n<- if(is.matrix(y)) nrow(y) else length(y)
     min.step<-opz$min.step
     rangeZ <- apply(Z, 2, range)
     alpha<-opz$alpha
-    limZ <- apply(Z, 2, quantile, names=FALSE, probs=c(alpha,1-alpha))
+    limZ <- apply(Z, 2, quantile, names=FALSE, probs=c(alpha[1],alpha[2]))
     psi<-PSI[1,]
+    psi<-adj.psi(psi, limZ)
+    PSI<-matrix(psi,nrow=n, ncol=ncol(PSI), byrow=TRUE)
     id.psi.group<-opz$id.psi.group
     conv.psi<-opz$conv.psi 
-    h<-opz$h
     digits<-opz$digits
     pow<-opz$pow
     nomiOK<-opz$nomiOK
     toll<-opz$toll
-    h<-opz$h
+    hh<-opz$h
     gap<-opz$gap
     #fix.npsi<-opz$fix.npsi
     fix.npsi<-opz$stop.if.error
@@ -111,8 +112,11 @@ seg.glm.fit<-function(y,XREG,Z,PSI,w,offs,opz,return.all.sol=FALSE){
     #    invXtX<- opz$invXtX
     #    Xty<-opz$Xty
     #===================
-    if(!in.psi(limZ,PSI,FALSE))  stop("starting psi out of the range.. see 'alpha' in seg.control", call.=FALSE)
-    if(!far.psi(Z,PSI,id.psi.group,FALSE)) stop("psi values too close each other. Please change (decreases number of) starting values", call.=FALSE)
+    
+    if(!in.psi(limZ, PSI,FALSE))  stop("starting psi out of the range.. see 'alpha' in seg.control", call.=FALSE)
+    if(!far.psi(Z,PSI,id.psi.group,FALSE)) 
+      stop("psi starting values too close each other or at the boundaries. Please change them (e.g. set 'quant=TRUE' 
+          in seg.control()), or decrease their number.", call. = FALSE)
     n.psi1<-ncol(Z)
     #==============================================
     U <- ((Z-PSI)*(Z>PSI)) #pmax((Z - PSI), 0)^pow[1]
@@ -121,15 +125,35 @@ seg.glm.fit<-function(y,XREG,Z,PSI,w,offs,opz,return.all.sol=FALSE){
                                      weights = w, family = fam, control = glm.control(maxit = maxit.glm), etastart = eta0))
     eta0<- obj0$linear.predictors
     L0<- obj0$dev
+    
+    
+    if(it.max==0){
+      colnames(U) <- paste("U", 1:ncol(U), sep = "")
+      V <- -(Z > PSI)
+      colnames(V) <- paste("V", 1:ncol(V), sep = "")
+      obj <- obj0 #lm.wfit(x = cbind(XREG, U), y = y, w = w, offset = offs)
+      L1 <- L0
+      obj$coefficients <- c(obj$coefficients, rep(0, ncol(V)))
+      #names(obj$coefficients) <- names.coef
+      obj$epsilon <- epsilon
+      obj$it <- it
+      obj <- list(obj = obj, it = it, psi = psi, psi.values = psi.values, 
+                  U = U, V = V, rangeZ = rangeZ, epsilon = epsilon, nomiOK = nomiOK, 
+                  dev.no.gap = L1, id.psi.group = id.psi.group, 
+                  id.warn = TRUE)
+      return(obj)
+    }
+    
     n.intDev0<-nchar(strsplit(as.character(L0),"\\.")[[1]][1])
     dev.values[length(dev.values) + 1] <- opz$dev0 #del modello iniziale (senza psi)
     dev.values[length(dev.values) + 1] <- L0 #modello con psi iniziali
     psi.values[[length(psi.values) + 1]] <- psi #psi iniziali
     #==============================================
+    #browser()
     if (visual) {
         cat(paste("iter = ", sprintf("%2.0f",0),
                   "  dev = ", sprintf(paste("%", n.intDev0+6, ".5f",sep=""), L0), #formatC(L1,width=8, digits=5,format="f"), #era format="fg" 
-                  "  k = ", sprintf("%2.0f", NA),
+                  "  k = ", sprintf("%5.0f", NA),
                   "  n.psi = ",formatC(length(unlist(psi)),digits=0,format="f"), 
                   "  ini.psi = ",paste(formatC(unlist(psi),digits=3,format="f"), collapse="  "), #sprintf('%.2f',x)
                   sep=""), "\n")
@@ -183,8 +207,11 @@ seg.glm.fit<-function(y,XREG,Z,PSI,w,offs,opz,return.all.sol=FALSE){
         }
         
         psi.old<-psi
-        psi <- psi.old + gamma.c/beta.c
-        psi<- adj.psi(psi, rangeZ)
+        psi <- psi.old + hh*gamma.c/beta.c
+        psi<- adj.psi(psi, limZ) #limZ or rangeZ???
+        
+        psi<-unlist(tapply(psi, opz$id.psi.group, sort), use.names =FALSE)
+        
         #############################aggiusta la stima di psi (nel range.. dopo in limZ)
         #DIREZIONE
         a<-optimize(search.min, c(0,1), psi=psi, psi.old=psi.old, X=XREG, y=y, w=w, offs=offs) #DUBBIO: Ma fam, eta0, L0 e maxit.glm devo passarli come argom,enti o li trova?
@@ -226,13 +253,13 @@ seg.glm.fit<-function(y,XREG,Z,PSI,w,offs,opz,return.all.sol=FALSE){
         #         break}
         # } #end while L0-L1
         
-        
+        #if(it==3) browser()
         
         if (visual) {
             flush.console()
             cat(paste("iter = ", sprintf("%2.0f",it),
                       "  dev = ", sprintf(paste("%", n.intDev0+6, ".5f",sep=""), L1), #formatC(L1,width=8, digits=5,format="f"), #era format="fg" 
-                      "  k = ", sprintf("%2.0f", use.k),
+                      "  k = ", sprintf("%2.3f", use.k),
                       "  n.psi = ",formatC(length(unlist(psi)),digits=0,format="f"), 
                       "  est.psi = ",paste(formatC(unlist(psi),digits=3,format="f"), collapse="  "), #sprintf('%.2f',x)
                       sep=""), "\n")
