@@ -3,9 +3,10 @@ seg.lm.fit <-function (y, XREG, Z, PSI, w, offs, opz, return.all.sol = FALSE)
     useExp.k = TRUE
     search.min<-function(h, psi, psi.old, X, y, w, offs) {
         psi.ok<- psi*h + psi.old*(1-h)
-        PSI <- matrix(rep(psi.ok, rep(n, length(psi.ok))), ncol = length(psi.ok))
+        #PSI <- matrix(rep(psi.ok, rep(n, length(psi.ok))), ncol = length(psi.ok))
+        PSI <- matrix(psi.ok, nrow=n, ncol = length(psi.ok), byrow=TRUE)
         U1 <- (Z - PSI) * (Z > PSI)
-        if (pow[1] != 1) U1 <- U1^pow[1]
+        #if (pow[1] != 1) U1 <- U1^pow[1]
         obj1 <- try(mylm(cbind(X, U1), y, w, offs), silent = TRUE)
         if (class(obj1)[1] == "try-error") obj1 <- try(lm.wfit(cbind(X, U1), y, w, offs), silent = TRUE)
         L1 <- if (class(obj1)[1] == "try-error") L0 + 10
@@ -52,8 +53,8 @@ seg.lm.fit <-function (y, XREG, Z, PSI, w, offs, opz, return.all.sol = FALSE)
         o
     }
     in.psi <- function(LIM, PSI, ret.id = TRUE) {
-        a <- PSI[1, ] <= LIM[1, ]
-        b <- PSI[1, ] >= LIM[2, ]
+        a <- PSI[1, ] < LIM[1, ]
+        b <- PSI[1, ] > LIM[2, ]
         is.ok <- !a & !b
         if (ret.id) 
             return(is.ok)
@@ -93,16 +94,20 @@ seg.lm.fit <-function (y, XREG, Z, PSI, w, offs, opz, return.all.sol = FALSE)
     min.step <- opz$min.step
     rangeZ <- apply(Z, 2, range)
     alpha <- opz$alpha
-    limZ <- apply(Z, 2, quantile, names = FALSE, probs = c(alpha, 1 - alpha))
+    limZ <- apply(Z, 2, quantile, names = FALSE, probs = c(alpha[1], alpha[2]))
+
+    #browser()
+    
     psi <- PSI[1, ]
+    psi<-adj.psi(psi, limZ)
+    PSI<-matrix(psi,nrow=n, ncol=ncol(PSI), byrow=TRUE)
     id.psi.group <- opz$id.psi.group
     conv.psi <- opz$conv.psi
-    h <- opz$h
+    hh <- opz$h
     digits <- opz$digits
     pow <- opz$pow
     nomiOK <- opz$nomiOK
     toll <- opz$toll
-    h <- opz$h
     gap <- opz$gap
     fix.npsi <- opz$stop.if.error
     dev.new <- opz$dev0
@@ -121,15 +126,37 @@ seg.lm.fit <-function (y, XREG, Z, PSI, w, offs, opz, return.all.sol = FALSE)
         XREG <- XREG[, sel.col.XREG, drop = FALSE]
     invXtX <- opz$invXtX
     Xty <- opz$Xty
+    
+    #browser()
     if (!in.psi(limZ, PSI, FALSE)) 
         stop("starting psi out of the range.. see 'alpha' in seg.control.", 
             call. = FALSE)
     if (!far.psi(Z, PSI, id.psi.group, FALSE)) 
-        stop("psi values too close each other. Please change (decreases number of) starting values", 
-            call. = FALSE)
+        stop("psi starting values too close each other or at the boundaries. Please change them (e.g. set 'quant=TRUE' 
+          in seg.control()), or decrease their number.", call. = FALSE)
     n.psi1 <- ncol(Z)
     U <- ((Z - PSI) * (Z > PSI))
     if (pow[1] != 1) U <- U^pow[1]
+    
+    if(it.max==0){
+      colnames(U) <- paste("U", 1:ncol(U), sep = "")
+      V <- -(Z > PSI)
+      colnames(V) <- paste("V", 1:ncol(V), sep = "")
+      obj <- lm.wfit(x = cbind(XREG, U), y = y, w = w, offset = offs)
+      L1 <- sum(obj$residuals^2 * w)
+      obj$coefficients <- c(obj$coefficients, rep(0, ncol(V)))
+      #names(obj$coefficients) <- names.coef
+      obj$epsilon <- epsilon
+      obj$it <- it
+      obj <- list(obj = obj, it = it, psi = psi, psi.values = psi.values, 
+                U = U, V = V, rangeZ = rangeZ, epsilon = epsilon, nomiOK = nomiOK, 
+                SumSquares.no.gap = L1, id.psi.group = id.psi.group, 
+                id.warn = TRUE)
+      return(obj)
+    }
+    
+    
+
     obj0 <- try(mylm(cbind(XREG, U), y, w, offs), silent = TRUE)
     if (class(obj0)[1] == "try-error") obj0 <- lm.wfit(cbind(XREG, U), y, w, offs)
     L0 <- sum(obj0$residuals^2 * w)
@@ -138,12 +165,12 @@ seg.lm.fit <-function (y, XREG, Z, PSI, w, offs, opz, return.all.sol = FALSE)
     dev.values[length(dev.values) + 1] <- L0
     psi.values[[length(psi.values) + 1]] <- psi
     if (visual) {
-        cat(paste("iter = ", sprintf("%2.0f", 0), "  dev = ", 
-            sprintf(paste("%", n.intDev0 + 6, ".5f", sep = ""), 
-                L0), "  k = ", sprintf("%2.0f", NA), "  n.psi = ", 
-            formatC(length(unlist(psi)), digits = 0, format = "f"), 
-            "  ini.psi = ", paste(formatC(unlist(psi), digits = 3, 
-                format = "f"), collapse = "  "), sep = ""), "\n")
+        cat(paste("iter = ", sprintf("%2.0f", 0), 
+                  "  dev = ",  sprintf(paste("%", n.intDev0 + 6, ".5f", sep = ""), L0), 
+                  "  k = ", sprintf("%5.0f", NA), 
+                  "  n.psi = ", formatC(length(unlist(psi)), digits = 0, format = "f"), 
+                  "  ini.psi = ", paste(formatC(unlist(psi), digits = 3, format = "f"), collapse = "  "), 
+                  sep = ""), "\n")
     }
     id.warn <- FALSE
     id.psi.changed <- rep(FALSE, it.max)
@@ -167,6 +194,9 @@ seg.lm.fit <-function (y, XREG, Z, PSI, w, offs, opz, return.all.sol = FALSE)
         colnames(X)[(ncol(XREG) + 1):ncol(X)] <- c(paste("U", 
             1:ncol(U), sep = ""), paste("V", 1:ncol(V), sep = ""))
         obj <- lm.wfit(x = X, y = y, w = w, offset = offs)
+        
+        #browser()
+        
         beta.c <- coef(obj)[paste("U", 1:ncol(U), sep = "")]
         gamma.c <- coef(obj)[paste("V", 1:ncol(V), sep = "")]
         if (any(is.na(c(beta.c, gamma.c)))) {
@@ -195,11 +225,14 @@ seg.lm.fit <-function (y, XREG, Z, PSI, w, offs, opz, return.all.sol = FALSE)
             }
         }
         psi.old <- psi
-        psi <- psi.old + gamma.c/beta.c
+        psi <- psi.old + hh*gamma.c/beta.c
         #aggiusta la stima di psi..
-        psi<- adj.psi(psi, rangeZ)
+        #psi<- adj.psi(psi, rangeZ) 
+        psi<- adj.psi(psi, limZ)
         #browser()
         
+        psi<-unlist(tapply(psi, opz$id.psi.group, sort), use.names =FALSE)
+        
         a<-optimize(search.min, c(0,1), psi=psi, psi.old=psi.old, X=XREG, y=y, w=w, offs=offs)
         k.values[length(k.values) + 1] <- use.k <- a$minimum
         L1<- a$objective
@@ -207,10 +240,11 @@ seg.lm.fit <-function (y, XREG, Z, PSI, w, offs, opz, return.all.sol = FALSE)
         psi <- psi*use.k + psi.old* (1-use.k)
         psi<- adj.psi(psi, limZ)
         if (!is.null(digits)) psi <- round(psi, digits)
-        PSI <- matrix(rep(psi, rep(n, length(psi))), ncol = length(psi))
+        #PSI <- matrix(rep(psi, rep(n, length(psi))), ncol = length(psi))
+        PSI <- matrix(psi, nrow=n, ncol = length(psi), byrow=TRUE)
         U1 <- (Z - PSI) * (Z > PSI)
         
-        
+        #if(it==3) browser()
         
         #if (pow[1] != 1) U1 <- U1^pow[1]
         #obj1 <- try(mylm(cbind(XREG, U1), y, w, offs), silent = TRUE)
@@ -241,8 +275,8 @@ seg.lm.fit <-function (y, XREG, Z, PSI, w, offs, opz, return.all.sol = FALSE)
             if (fix.npsi) {
                 psi <- psi * ifelse(id.psi.far, 1, attr(id.psi.far, 
                   "factor"))
-                PSI <- matrix(rep(psi, rep(nrow(Z), length(psi))), 
-                  ncol = length(psi))
+                #PSI <- matrix(rep(psi, rep(nrow(Z), length(psi))), ncol = length(psi))
+                PSI <- matrix(psi, nrow=n, ncol = length(psi), byrow=TRUE)
                 id.psi.changed[it] <- TRUE
             }
             else {
@@ -280,14 +314,15 @@ seg.lm.fit <-function (y, XREG, Z, PSI, w, offs, opz, return.all.sol = FALSE)
     names(psi) <- id.psi.group
     names.coef <- names(obj$coefficients)
     #PSI.old <- PSI
-    PSI <- matrix(rep(psi, rep(nrow(Z), length(psi))), ncol = length(psi))
+    #PSI <- matrix(rep(psi, rep(nrow(Z), length(psi))), ncol = length(psi))
+    PSI <- matrix(psi, nrow=n, ncol = length(psi), byrow=TRUE)
     #if (sd(PSI - PSI.old) > 0 || id.psi.changed[length(id.psi.changed)]) {
-        U <- (Z - PSI) * (Z > PSI)
-        colnames(U) <- paste("U", 1:ncol(U), sep = "")
-        V <- -(Z > PSI)
-        colnames(V) <- paste("V", 1:ncol(V), sep = "")
-        obj <- lm.wfit(x = cbind(XREG, U), y = y, w = w, offset = offs)
-        L1 <- sum(obj$residuals^2 * w)
+    U <- (Z - PSI) * (Z > PSI)
+    colnames(U) <- paste("U", 1:ncol(U), sep = "")
+    V <- -(Z > PSI)
+    colnames(V) <- paste("V", 1:ncol(V), sep = "")
+    obj <- lm.wfit(x = cbind(XREG, U), y = y, w = w, offset = offs)
+    L1 <- sum(obj$residuals^2 * w)
     #}
     #else {
     #    obj <- obj1
