@@ -38,8 +38,11 @@ function(obj, seg.Z, psi, npsi, fixed.psi=NULL, control = seg.control(), model =
     #   }
     # if(length(all.vars(seg.Z))!=n.Seg) stop("A wrong number of terms in `seg.Z' or `psi'")
   if(missing(seg.Z)) {
-    if(length(all.vars(formula(obj)))==2) seg.Z<- as.formula(paste("~", all.vars(formula(obj))[2])) else stop("please specify 'seg.Z'")
+    nomeX<- intersect(paste(obj$call$xreg), names(obj$coef))
+    if(length(nomeX)==1) seg.Z<- as.formula(paste("~",nomeX)) else stop("please specify 'seg.Z'")
+    #if(length(all.vars(formula(obj)))==2) seg.Z<- as.formula(paste("~", all.vars(formula(obj))[2])) else stop("please specify 'seg.Z'")
   }
+  #browser()
   if("V" %in% sub("V[1-9]*[0-9]","V", c(all.vars(seg.Z), names(coef(obj))))) stop("variable names 'V', 'V1', .. are not allowed")
   if("U" %in% sub("U[1-9]*[0-9]","U", c(all.vars(seg.Z), names(coef(obj))))) stop("variable names 'U', 'U1', .. are not allowed")
   if(any(c("$","[") %in% all.names(seg.Z))) stop(" '$' or '[' not allowed in 'seg.Z' ")
@@ -80,7 +83,6 @@ function(obj, seg.Z, psi, npsi, fixed.psi=NULL, control = seg.control(), model =
     digits<-control$digits
     toll <- control$toll
     if(toll<0) stop("Negative tolerance ('tol' in seg.control()) is meaningless", call. = FALSE)
-    visual <- control$visual
     stop.if.error<-control$stop.if.error
     fix.npsi<-fix.npsi<-control$fix.npsi
     if(!is.null(stop.if.error)) {#if the old "stop.if.error" has been used..
@@ -95,18 +97,20 @@ function(obj, seg.Z, psi, npsi, fixed.psi=NULL, control = seg.control(), model =
     random<-control$random
     pow<-control$pow
     conv.psi<-control$conv.psi
+    visual <- control$visual
     visualBoot<-FALSE
-    if(n.boot>0){
-      if(!is.null(control$seed)) {
-        set.seed(control$seed)
-        employed.Random.seed<-control$seed
-      } else {
-        employed.Random.seed<-eval(parse(text=paste(sample(0:9, size=6), collapse="")))
-        set.seed(employed.Random.seed)
-      }
-      if(visual) {visual<-FALSE; visualBoot<-TRUE}# warning("`display' set to FALSE with bootstrap restart", call.=FALSE)}
-      if(!stop.if.error) stop("Bootstrap restart only with a fixed number of breakpoints")
-    }
+    if(visual && n.boot>0) {visual<-FALSE; visualBoot<-TRUE}
+    # if(n.boot>0){
+    #   if(!is.null(control$seed)) {
+    #     set.seed(control$seed)
+    #     employed.Random.seed<-control$seed
+    #   } else {
+    #     employed.Random.seed<-eval(parse(text=paste(sample(0:9, size=6), collapse="")))
+    #     set.seed(employed.Random.seed)
+    #   }
+    #   if(visual) {visual<-FALSE; visualBoot<-TRUE}# warning("`display' set to FALSE with bootstrap restart", call.=FALSE)}
+    #   if(!stop.if.error) stop("Bootstrap restart only with a fixed number of breakpoints")
+    # }
     last <- control$last
     K<-control$K
     h<-control$h
@@ -227,9 +231,12 @@ function(obj, seg.Z, psi, npsi, fixed.psi=NULL, control = seg.control(), model =
     list.obj <- list(obj)
     nomiOK<-nomiU
     
+    if(is.null(alpha)) alpha<- max(.05, 1/nrow(PSI))
+    if(length(alpha)==1) alpha<-c(alpha, 1-alpha)
+    
     opz<-list(toll=toll,h=h,stop.if.error=stop.if.error,dev0=dev0,visual=visual,it.max=it.max,
               nomiOK=nomiOK, id.psi.group=id.psi.group, gap=gap, visualBoot=visualBoot, pow=pow, digits=digits,
-              conv.psi=conv.psi, alpha=alpha, fix.npsi=fix.npsi, min.step=min.step,fc=fc)
+              conv.psi=conv.psi, alpha=alpha, fix.npsi=fix.npsi, min.step=min.step, fc=fc, seed=control$seed)
     
     opz$call.ok<-call.ok
     opz$call.noV<-call.noV
@@ -237,9 +244,11 @@ function(obj, seg.Z, psi, npsi, fixed.psi=NULL, control = seg.control(), model =
     opz$nomiV<-nomiV
 
     if(n.boot<=0){
-      obj<-seg.Ar.fit(obj, XREG, Z, PSI, opz)
+      obj<- seg.Ar.fit(obj, XREG, Z, PSI, opz)
     } else {
-      obj<-seg.Ar.fit.boot(obj, XREG, Z, PSI, opz, n.boot=n.boot, size.boot=size.boot, random=random, break.boot=break.boot) #jt, nonParam
+      obj<- seg.Ar.fit.boot(obj, XREG, Z, PSI, opz, n.boot=n.boot, size.boot=size.boot, random=random, 
+                            break.boot=break.boot) #jt, nonParam
+      seed <- obj$seed
     }
     
     if(!is.list(obj)){
@@ -345,7 +354,8 @@ function(obj, seg.Z, psi, npsi, fixed.psi=NULL, control = seg.control(), model =
     objF$id.warn <- id.warn
     ###########################PSI FIXED
     objF$indexU<-build.all.psi(psi.list, fixed.psi)
-    if(n.boot>0) objF$seed<-employed.Random.seed
+    objF$psi[,"Initial"]<-NA
+    if(n.boot>0) objF$seed<-seed
     class(objF) <- c("segmented", class(obj0))
     list.obj[[length(list.obj) + 1]] <- objF
     class(list.obj) <- "segmented"
