@@ -117,7 +117,6 @@ segmented.default<-function (obj, seg.Z, psi, npsi, fixed.psi=NULL, control = se
     digits <- control$digits
     toll <- control$toll
     if (toll < 0)  stop("Negative tolerance ('tol' in seg.control()) is meaningless", call. = FALSE)
-    visual <- control$visual
     stop.if.error <- fix.npsi <- control$fix.npsi
     break.boot=control$break.boot
     n.boot <- control$n.boot
@@ -126,23 +125,25 @@ segmented.default<-function (obj, seg.Z, psi, npsi, fixed.psi=NULL, control = se
     random <- control$random
     pow <- control$pow
     conv.psi <- control$conv.psi
-    visualBoot <- FALSE
-    if (n.boot > 0) {
-        if (!is.null(control$seed)) {
-            set.seed(control$seed)
-            employed.Random.seed <- control$seed
-        }
-        else {
-            employed.Random.seed <- eval(parse(text = paste(sample(0:9, 
-                size = 6), collapse = "")))
-            set.seed(employed.Random.seed)
-        }
-        if (visual) {
-            visual <- FALSE
-            visualBoot <- TRUE
-        }
-        if (!stop.if.error) stop("Bootstrap restart only with a fixed number of breakpoints")
-    }
+    visual <- control$visual
+    visualBoot<-FALSE
+    if(visual && n.boot>0) {visual<-FALSE; visualBoot<-TRUE}
+    # if (n.boot > 0) {
+    #     if (!is.null(control$seed)) {
+    #         set.seed(control$seed)
+    #         employed.Random.seed <- control$seed
+    #     }
+    #     else {
+    #         employed.Random.seed <- eval(parse(text = paste(sample(0:9, 
+    #             size = 6), collapse = "")))
+    #         set.seed(employed.Random.seed)
+    #     }
+    #     if (visual) {
+    #         visual <- FALSE
+    #         visualBoot <- TRUE
+    #     }
+    #     if (!stop.if.error) stop("Bootstrap restart only with a fixed number of breakpoints")
+    # }
     last <- control$last
     K <- control$K
     h <- control$h
@@ -168,9 +169,20 @@ segmented.default<-function (obj, seg.Z, psi, npsi, fixed.psi=NULL, control = se
                 collapse = "+")))
     }
     
-    if(!is.null(obj$call$random)) mfExt$formula<-update.formula(mf$formula, paste(".~.+", paste(all.vars(obj$call$random), collapse="+")))
+    if(!is.null(obj$call$random)) {
+      nomiRandom <- all.vars(obj$call$random)
+      if(is.list(eval(obj$call$random))) nomiRandom <- c(nomiRandom, names(eval(obj$call$random))) 
+      mfExt$formula<-update.formula(mf$formula, paste(".~.+", paste(nomiRandom, collapse="+")))
+    }
     
-    mf <- eval(mf, parent.frame())
+    if (inherits(obj, "svyglm")) {
+      mydesign <- eval(obj$call$design)
+      mf$data=quote(mydesign$variables)
+      mfExt$data=quote(mydesign$variables)
+      mf<- eval(mf)
+    } else {
+      mf <- eval(mf, parent.frame())
+    }
     n <- nrow(mf)
     nomiOff <- setdiff(all.vars(formula(obj)), names(mf))
     if (length(nomiOff) >= 1) 
@@ -186,7 +198,12 @@ segmented.default<-function (obj, seg.Z, psi, npsi, fixed.psi=NULL, control = se
     if (!is.null(nomiNO)) 
         mfExt$formula <- update.formula(mfExt$formula, paste(".~.-", 
             paste(nomiNO, collapse = "-"), sep = ""))
-    mfExt <- eval(mfExt, parent.frame())
+    
+    if(inherits(obj, "svyglm")){
+      mfExt <- eval(mfExt)
+    } else {
+      mfExt <- eval(mfExt, parent.frame())
+    }
     if (inherits(obj, "coxph")) {
         is.Surv <- NA
         rm(is.Surv)
@@ -306,6 +323,19 @@ segmented.default<-function (obj, seg.Z, psi, npsi, fixed.psi=NULL, control = se
     
     call.ok <- update(obj, Fo, evaluate = FALSE, data = mfExt)
     call.noV <- update(obj, Fo.noV, evaluate = FALSE, data = mfExt)
+    
+    #browser()
+    
+    if (inherits(obj, "svyglm")){
+      #mydesign <- eval(obj$call$design)
+      for (i in 1:k) {
+        mydesign$variables[nomiU[i]] <- U[, i]
+        mydesign$variables[nomiV[i]] <- V[, i]
+      }
+      call.ok$design<- call.noV$design<-quote(mydesign)
+      call.ok$data<-call.noV$data<-NULL
+    }
+    
     if (it.max == 0) {
         if (!is.null(call.noV[["subset"]])) 
             call.noV[["subset"]] <- NULL
@@ -324,11 +354,12 @@ segmented.default<-function (obj, seg.Z, psi, npsi, fixed.psi=NULL, control = se
     list.obj <- list(obj)
     nomiOK <- nomiU
     if(is.null(alpha)) alpha<- max(.05, 1/nrow(PSI))
+    if(length(alpha)==1) alpha<-c(alpha, 1-alpha)
     opz <- list(toll = toll, h = h, stop.if.error = stop.if.error, 
         dev0 = dev0, visual = visual, it.max = it.max, nomiOK = nomiOK, 
         id.psi.group = id.psi.group, gap = gap, visualBoot = visualBoot, 
         pow = pow, digits = digits, conv.psi = conv.psi, alpha = alpha, 
-        fix.npsi = fix.npsi, min.step = min.step)
+        fix.npsi = fix.npsi, min.step = min.step, seed=control$seed)
     opz$call.ok <- call.ok
     opz$call.noV <- call.noV
     opz$formula.orig <- formula(obj)
@@ -336,13 +367,14 @@ segmented.default<-function (obj, seg.Z, psi, npsi, fixed.psi=NULL, control = se
     opz$nomiV <- nomiV
     opz$fn.obj <- fn.obj
     opz$fc=fc
+    if (inherits(obj, "svyglm")) opz$mydesign <- mydesign
     opz <- c(opz, ...)
     if (n.boot <= 0) {
         obj <- seg.def.fit(obj, Z, PSI, mfExt, opz)
-    }
-    else {
+    } else {
         obj <- seg.def.fit.boot(obj, Z, PSI, mfExt, opz, n.boot = n.boot, 
             size.boot = size.boot, random = random, break.boot=break.boot)
+        seed<- obj$seed
     }
     
     if (!is.list(obj)) {
@@ -403,14 +435,19 @@ segmented.default<-function (obj, seg.Z, psi, npsi, fixed.psi=NULL, control = se
     beta.c<-unlist( unique(coef(obj)[nomiU])) #beta.c<-coef(obj)[nomiU]
     Vxb <- V %*% diag(beta.c, ncol = length(beta.c))
     nnomi <- c(nomiU, nomiVxb)
-    for (i in 1:ncol(U)) {
-        mfExt[nomiU[i]] <- mf[nomiU[i]] <- U[, i]
-        mfExt[nomiVxb[i]] <- mf[nomiVxb[i]] <- Vxb[, i]
-    }
     Fo <- update.formula1(formula(obj0), as.formula(paste(".~.+", paste(nnomi, collapse = "+"))), opt=1)
     ###############
-
-
+    if (inherits(obj, "svyglm")){
+      for (i in 1:ncol(U)) {
+        mydesign$variables[nomiU[i]] <- U[, i]
+        mydesign$variables[nomiVxb[i]] <- Vxb[, i]
+      }
+    } else {
+      for (i in 1:ncol(U)) {
+        mfExt[nomiU[i]] <- mf[nomiU[i]] <- U[, i]
+        mfExt[nomiVxb[i]] <- mf[nomiVxb[i]] <- Vxb[, i]
+      }
+    }
 
 
 
@@ -431,7 +468,23 @@ segmented.default<-function (obj, seg.Z, psi, npsi, fixed.psi=NULL, control = se
         objF$R <- quote(R)
         objF$r <- quote(r)
     }
+    
+    #browser()
+    
+    if (inherits(obj, "svyglm")){
+      objF$design<- call.noV$design<-quote(mydesign)
+      objF$data<-call.noV$data<-NULL
+    }
+    
     objF <- eval(objF, envir = mfExt)
+    
+    
+    
+    
+    
+    
+    
+    
     objF$offset <- obj0$offset
     isNAcoef <- any(is.na(coef(objF)))
     if (isNAcoef) {
@@ -517,12 +570,13 @@ segmented.default<-function (obj, seg.Z, psi, npsi, fixed.psi=NULL, control = se
     }
     else {
         vv <- Cov[nomiVxb, nomiVxb, drop=FALSE]
+        vv<-sqrt(diag(vv))
     }
     ris.psi <- matrix(NA, length(psi), 3)
     colnames(ris.psi) <- c("Initial", "Est.", "St.Err")
     rownames(ris.psi) <- nomiVxb
     ris.psi[, 2] <- psi
-    ris.psi[, 3] <- sqrt(diag(vv))
+    ris.psi[, 3] <- vv
     a <- tapply(id.psi.group, id.psi.group, length)
     a.ok <- NULL
     for (j in name.Z) {
@@ -558,15 +612,14 @@ segmented.default<-function (obj, seg.Z, psi, npsi, fixed.psi=NULL, control = se
     objF$indexU<-build.all.psi(psi.list, fixed.psi)
     ##############
     if (model) objF$model <- mf
-    if (n.boot > 0) 
-        objF$seed <- employed.Random.seed
-    if (keep.class) 
-        class(objF) <- c("segmented", class(obj0))
+    if (n.boot > 0) objF$seed <- seed
+    if (keep.class) class(objF) <- c("segmented", class(obj0))
+    objF$psi[,"Initial"]<-NA
     list.obj[[length(list.obj) + 1]] <- objF
     class(list.obj) <- "segmented"
     if (last) 
         list.obj <- list.obj[[length(list.obj)]]
-    warning("The returned fit is ok, but not of class 'segmented'. If interested, call explicitly the segmented methods (plot.segmented, confint.segmented,..)", 
+    warning("The returned fit is OK, but not of class 'segmented'.\n If interested, call explicitly the segmented methods (plot.segmented, confint.segmented,..)", 
         call. = FALSE)
     return(list.obj)
 }
