@@ -63,7 +63,7 @@ function(obj, seg.Z, psi, npsi, fixed.psi=NULL, control = seg.control(), model =
     digits<-control$digits
     toll <- control$toll
     if(toll<0) stop("Negative tolerance ('tol' in seg.control()) is meaningless", call. = FALSE)
-    visual <- control$visual
+    
     stop.if.error<-control$stop.if.error
     fix.npsi<-fix.npsi<-control$fix.npsi
     if(!is.null(stop.if.error)) {#if the old "stop.if.error" has been used..
@@ -78,18 +78,20 @@ function(obj, seg.Z, psi, npsi, fixed.psi=NULL, control = seg.control(), model =
     random<-control$random
     pow<-control$pow
     conv.psi<-control$conv.psi
+    visual <- control$visual
     visualBoot<-FALSE
-    if(n.boot>0){
-      if(!is.null(control$seed)) {
-        set.seed(control$seed)
-        employed.Random.seed<-control$seed
-      } else {
-        employed.Random.seed<-eval(parse(text=paste(sample(0:9, size=6), collapse="")))
-        set.seed(employed.Random.seed)
-      }
-      if(visual) {visual<-FALSE; visualBoot<-TRUE}#warning("`display' set to FALSE with bootstrap restart", call.=FALSE)}
-      if(!stop.if.error) stop("Bootstrap restart only with a fixed number of breakpoints")
-    }
+    if(visual && n.boot>0) {visual<-FALSE; visualBoot<-TRUE}
+    # if(n.boot>0){
+    #   if(!is.null(control$seed)) {
+    #     set.seed(control$seed)
+    #     employed.Random.seed<-control$seed
+    #   } else {
+    #     employed.Random.seed<-eval(parse(text=paste(sample(0:9, size=6), collapse="")))
+    #     set.seed(employed.Random.seed)
+    #   }
+    #   if(visual) {visual<-FALSE; visualBoot<-TRUE}#warning("`display' set to FALSE with bootstrap restart", call.=FALSE)}
+    #   if(!stop.if.error) stop("Bootstrap restart only with a fixed number of breakpoints")
+    # }
     last <- control$last
     K<-control$K
     h<-min(abs(control$h),1)
@@ -101,7 +103,7 @@ function(obj, seg.Z, psi, npsi, fixed.psi=NULL, control = seg.control(), model =
     mf$drop.unused.levels <- TRUE
     mf[[1L]] <- as.name("model.frame")
     #non so a che serva la seguente linea..
-    if(class(mf$formula)=="name" && !"~"%in%paste(mf$formula)) mf$formula<-eval(mf$formula)
+    if(class(mf$formula)[1]=="name" && !"~"%in%paste(mf$formula)) mf$formula<-eval(mf$formula)
     mfExt<- mf
     mf$formula<-update.formula(mf$formula,paste(seg.Z,collapse=".+"))
     if(!is.null(obj$call$offset) || !is.null(obj$call$weights) || !is.null(obj$call$subset)){ 
@@ -114,7 +116,9 @@ function(obj, seg.Z, psi, npsi, fixed.psi=NULL, control = seg.control(), model =
                          collapse = "+")
                        ))
     }
-    mf <-  eval(mf, parent.frame())
+    #browser()
+    mf <-  eval(mf, parent.frame()) 
+    #mf <-  eval(mf, parent.frame(max(1,sys.parent())))
     n<-nrow(mf)
     #La linea sotto serve per inserire in mfExt le eventuali variabili contenute nella formula con offset(..)
     #   o anche variabili che rientrano in espressioni (ad es., y/n o I(y*n))
@@ -125,14 +129,14 @@ function(obj, seg.Z, psi, npsi, fixed.psi=NULL, control = seg.control(), model =
     nomiTUTTI<-all.vars(mfExt$formula) #comprende anche altri nomi (ad es., threshold) "variabili"
     nomiNO<-NULL #dovrebbe contenere
     for(i in nomiTUTTI){
-      r<-try(eval(parse(text=i), parent.frame()), silent=TRUE)
-      if(class(r)!="try-error" && length(r)==1 && !is.function(r) && !i%in%names(mf)) nomiNO[[length(nomiNO)+1]]<-i
+      r<-try(eval(parse(text=i), parent.frame()), silent=TRUE) # parent.frame(max(1,sys.parent())))
+      if(class(r)[1]!="try-error" && length(r)==1 && !is.function(r) && !i%in%names(mf)) nomiNO[[length(nomiNO)+1]]<-i
     }
     #nomiNO dovrebbe contenere i nomi delle "altre variabili" (come th in subset=x<th) 
     if(!is.null(nomiNO)) mfExt$formula<-update.formula(mfExt$formula,paste(".~.-", paste( nomiNO, collapse="-"), sep=""))
     
-    mfExt<-eval(mfExt, parent.frame())
     
+    mfExt<-eval(mfExt, parent.frame()) #mfExt<-eval(mfExt, parent.frame(max(1,sys.parent())))
     #id.offs<-pmatch("offset",names(mf)) #questa identifica il nome offset(..). ELiminarlo dal dataframe? non conviene
     #       altrimenti nel model.frame non risulta l'offset
     
@@ -268,21 +272,25 @@ function(obj, seg.Z, psi, npsi, fixed.psi=NULL, control = seg.control(), model =
     dev0<-obj$dev
     list.obj <- list(obj)
     nomiOK<-nomiU
-    opz<-list(toll=toll,h=h,stop.if.error=stop.if.error,dev0=dev0,visual=visual,it.max=it.max,nomiOK=nomiOK,
+    if(is.null(alpha)) alpha<- max(.05, 1/length(y))
+    if(length(alpha)==1) alpha<-c(alpha, 1-alpha)
+    opz<-list(toll=toll, h=h, stop.if.error=stop.if.error, dev0=dev0, visual=visual, it.max=it.max, nomiOK=nomiOK,
         fam=fam, eta0=obj$linear.predictors, maxit.glm=maxit.glm, id.psi.group=id.psi.group, gap=gap,
         conv.psi=conv.psi, alpha=alpha, fix.npsi=fix.npsi, min.step=min.step,
-        pow=pow, visualBoot=visualBoot, digits=digits, fc=fc)   
+        pow=pow, visualBoot=visualBoot, digits=digits, fc=fc, seed=control$seed)   
 
+    #browser()
+    
     if(n.boot<=0){
-      obj<-seg.glm.fit(y,XREG,Z,PSI,weights,offs,opz)
+      obj<-seg.glm.fit(y, XREG, Z, PSI, weights, offs, opz)
     } else {
       obj<-seg.glm.fit.boot(y, XREG, Z, PSI, weights, offs, opz, n.boot=n.boot, size.boot=size.boot, random=random, break.boot=break.boot) #jt, nonParam
-      }
+      seed<- obj$seed
+    }
     if(!is.list(obj)){
         warning("No breakpoint estimated", call. = FALSE)
         return(obj0)
         }
-    
     id.psi.group<-obj$id.psi.group
     nomiOK<-obj$nomiOK
     nomiFINALI<-unique(sub("U[1-9]*[0-9].", "", nomiOK)) #nomi originali delle variabili con breakpoint stimati!
@@ -433,7 +441,8 @@ function(obj, seg.Z, psi, npsi, fixed.psi=NULL, control = seg.control(), model =
     ###########################PSI FIXED
     objF$indexU<-build.all.psi(psi.list, fixed.psi)
     if (model)  objF$model <- mf #objF$mframe <- data.frame(as.list(KK))
-    if(n.boot>0) objF$seed<-employed.Random.seed
+    if(n.boot>0) objF$seed<- seed #employed.Random.seed
+    objF$psi[,"Initial"]<-NA
     class(objF) <- c("segmented", class(obj0))
     list.obj[[length(list.obj) + 1]] <- objF
     class(list.obj) <- "segmented"
