@@ -1,6 +1,6 @@
 `segmented.lm` <-
 function(obj, seg.Z, psi, npsi, fixed.psi=NULL, control = seg.control(), model = TRUE, keep.class=FALSE, ...) {
-  
+
   build.all.psi<-function(psi, fixed.psi){
     all.names.psi<-union(names(psi),names(fixed.psi))
     all.psi<-vector("list", length=length(all.names.psi))
@@ -65,7 +65,7 @@ function(obj, seg.Z, psi, npsi, fixed.psi=NULL, control = seg.control(), model =
     digits<-control$digits
     toll <- control$toll
     if(toll<0) stop("Negative tolerance ('tol' in seg.control()) is meaningless", call. = FALSE)
-    visual <- control$visual
+    
     
     stop.if.error<-control$stop.if.error
     fix.npsi<-fix.npsi<-control$fix.npsi
@@ -82,18 +82,20 @@ function(obj, seg.Z, psi, npsi, fixed.psi=NULL, control = seg.control(), model =
     random<-control$random
     pow<-control$pow
     conv.psi<-control$conv.psi
+    visual <- control$visual
     visualBoot<-FALSE
-    if(n.boot>0){
-        if(!is.null(control$seed)) {
-            set.seed(control$seed)
-            employed.Random.seed<-control$seed
-              } else {
-            employed.Random.seed<-eval(parse(text=paste(sample(0:9, size=6), collapse="")))
-            set.seed(employed.Random.seed)
-              }
-        if(visual) {visual<-FALSE; visualBoot<-TRUE}# warning("`display' set to FALSE with bootstrap restart", call.=FALSE)}
-#        if(!stop.if.error) stop("Bootstrap restart only with a fixed number of breakpoints")
-     }
+    if(visual && n.boot>0) {visual<-FALSE; visualBoot<-TRUE}
+    # if(n.boot>0){
+    #     if(!is.null(control$seed)) {
+    #         set.seed(control$seed)
+    #         employed.Random.seed<-control$seed
+    #           } else {
+    #         employed.Random.seed<-eval(parse(text=paste(sample(0:9, size=6), collapse="")))
+    #         set.seed(employed.Random.seed)
+    #           }
+    #     if(visual) {visual<-FALSE; visualBoot<-TRUE}# warning("`display' set to FALSE with bootstrap restart", call.=FALSE)}
+    # }
+    
     last <- control$last
     K<-control$K
     h<-control$h
@@ -104,7 +106,7 @@ function(obj, seg.Z, psi, npsi, fixed.psi=NULL, control = seg.control(), model =
     mf <- mf[c(1, m)]
     mf$drop.unused.levels <- TRUE
     mf[[1L]] <- as.name("model.frame")
-    if(class(mf$formula)=="name" && !"~"%in%paste(mf$formula)) mf$formula<-eval(mf$formula)
+    if(class(mf$formula)[1]=="name" && !"~"%in%paste(mf$formula)) mf$formula<-eval(mf$formula)
     #orig.call$formula<-update.formula(orig.call$formula, paste("~.-",all.vars(seg.Z)))
 #
 #genn 2013. dalla versione 0.2.9-4 ho tolto if(length(.. Tra l'altro non capisco perche' lo avevo fatto
@@ -134,7 +136,13 @@ function(obj, seg.Z, psi, npsi, fixed.psi=NULL, control = seg.control(), model =
             collapse = "+")
             ))
           }
-    mf <-  eval(mf, parent.frame())
+    
+    #browser()
+    #mf <-  try(eval(mf, parent.frame(max(1,sys.parent()))), silent=TRUE)
+    #if(inherits(mf, "try-error")) mf <-  try(eval(mf, parent.frame()), silent=TRUE)
+    # 
+    mf <- eval(mf, parent.frame())
+    
     n<-nrow(mf)
     #questo serve per inserire in mfExt le eventuali variabili contenute nella formula con offset(..)
     nomiOff<-setdiff(all.vars(formula(obj)), names(mf))
@@ -144,8 +152,12 @@ function(obj, seg.Z, psi, npsi, fixed.psi=NULL, control = seg.control(), model =
 nomiTUTTI<-all.vars(mfExt$formula) #comprende anche altri nomi (ad es., threshold) "variabili"
 nomiNO<-NULL 
 for(i in nomiTUTTI){
-    r<-try(eval(parse(text=i), parent.frame()), silent=TRUE)
-    if(class(r)!="try-error" && length(r)==1 && !is.function(r) && !i%in%names(mf)) nomiNO[[length(nomiNO)+1]]<-i
+  #r<-try(eval(parse(text=i), parent.frame(max(1,sys.parent()))), silent=TRUE)  
+  #
+    #if(inherits(r,"try-error")) 
+      r<-try(eval(parse(text=i), parent.frame()), silent=TRUE) 
+      
+    if(class(r)[1]!="try-error" && length(r)==1 && !is.function(r) && !i%in%names(mf)) nomiNO[[length(nomiNO)+1]]<-i
     }
 #nomiNO dovrebbe contenere i nomi delle "altre variabili" (come th in subset=x<th) 
 if(!is.null(nomiNO)) mfExt$formula<-update.formula(mfExt$formula,paste(".~.-", paste( nomiNO, collapse="-"), sep=""))
@@ -157,7 +169,14 @@ if(!is.null(nomiNO)) mfExt$formula<-update.formula(mfExt$formula,paste(".~.-", p
     #nomiAgg<- setdiff(names(mf), names(mfExt))
     #if(length(nomiAgg)>0) mfExt$formula<-update.formula(mfExt$formula,paste(".~.+", paste( nomiAgg, collapse="+"), sep=""))
     
-    mfExt<-eval(mfExt, parent.frame())
+#
+  # 
+#    mfExt <-  try(eval(mfExt, parent.frame(max(1,sys.parent()))), silent=TRUE)
+#    if(inherits(mfExt, "try-error")) mfExt <-  try(eval(mfExt, parent.frame()), silent=TRUE)
+mfExt <-eval(mfExt, parent.frame())
+#    mfExt <- eval(mfExt, parent.frame(max(1,sys.parent())))
+
+
     
     #mantieni in mfExt solo le variabili che NON ci sono in mf (cosi la funzione occupa meno spazio..)
     #mfExt<-mfExt[,setdiff(names(mfExt), names(mf)),drop=FALSE]
@@ -200,6 +219,8 @@ if(!is.null(nomiNO)) mfExt$formula<-update.formula(mfExt$formula,paste(".~.-", p
     #################
     #if(ncol(XREGseg)==1 && length(psi)==1 && n.psi==1 && !any(is.na(psi))) { if(psi==Inf) psi<-median(XREGseg)}
     #################
+    #browser()
+
     n <- nrow(XREG)
     Z<-lapply(apply(XREGseg,2,list),unlist) #prende anche i nomi!
     name.Z <- names(Z) <- colnames(XREGseg)
@@ -222,7 +243,9 @@ if(!is.null(nomiNO)) mfExt$formula<-update.formula(mfExt$formula,paste(".~.-", p
         if(any(is.na(psi[[i]]))) psi[[i]]<-if(control$quant) {quantile(Z[[i]], prob= seq(0,1,l=K+2)[-c(1,K+2)], names=FALSE)} else {(min(Z[[i]])+ diff(range(Z[[i]]))*(1:K)/(K+1))}
         }
     }
-
+  
+    
+    
   #########==================== SE PSI FIXED
   id.psi.fixed <- FALSE
   if(!is.null(fixed.psi)){
@@ -313,21 +336,29 @@ if(!is.null(nomiNO)) mfExt$formula<-update.formula(mfExt$formula,paste(".~.-", p
     list.obj <- list(obj)
 #    psi.values <- NULL
     nomiOK<-nomiU
-    invXtX<-if(!is.null(obj$qr)) chol2inv(qr.R(obj$qr)) else NULL #(XtX)^{-1}
-    Xty<-crossprod(XREG,y)
+    #invXtX<-if(!is.null(obj$qr)) chol2inv(qr.R(obj$qr)) else NULL #(XtX)^{-1}
+    #Xty<-crossprod(XREG,y)
+    invXtX<-Xty<-NULL
+    #browser()
+    if(is.null(alpha)) alpha<- max(.05, 1/length(y))
+    if(length(alpha)==1) alpha<-c(alpha, 1-alpha)
     opz<-list(toll=toll,h=h, stop.if.error=stop.if.error, dev0=dev0, visual=visual, it.max=it.max,
         nomiOK=nomiOK, id.psi.group=id.psi.group, gap=gap, visualBoot=visualBoot, pow=pow, digits=digits,invXtX=invXtX, Xty=Xty, 
-        conv.psi=conv.psi, alpha=alpha, fix.npsi=fix.npsi, min.step=min.step, fc=fc)
+        conv.psi=conv.psi, alpha=alpha, fix.npsi=fix.npsi, min.step=min.step, fc=fc, seed=control$seed)
     if(n.boot<=0){
     obj<-seg.lm.fit(y,XREG,Z,PSI,weights,offs,opz)
     } else {
-    obj<-seg.lm.fit.boot(y, XREG, Z, PSI, weights, offs, opz, n.boot=n.boot, size.boot=size.boot, random=random, break.boot=break.boot) #jt, nonParam
+    obj<-seg.lm.fit.boot(y, XREG, Z, PSI, weights, offs, opz, n.boot=n.boot, size.boot=size.boot, random=random, break.boot=break.boot)
+    seed<- obj$seed
       }
     if(!is.list(obj)){
         warning("No breakpoint estimated", call. = FALSE)
         return(obj0)
         }
     
+    #browser()
+    
+    
     if(obj$obj$df.residual==0) warning("no residual degrees of freedom (other warnings expected)", call.=FALSE)
     id.psi.group<-obj$id.psi.group
     nomiOK<-obj$nomiOK
@@ -354,9 +385,6 @@ if(!is.null(nomiNO)) mfExt$formula<-update.formula(mfExt$formula,paste(".~.-", p
     length.psi<-tapply(as.numeric(as.character(names(psi))), as.numeric(as.character(names(psi))), length)
     forma.nomiU<-function(xx,yy)paste("U",1:xx, ".", yy, sep="")
     forma.nomiVxb<-function(xx,yy)paste("psi",1:xx, ".", yy, sep="")
-    
-    #nomiU   <- unlist(mapply(forma.nomiU, length.psi, name.Z)) #invece di un ciclo #paste("U",1:length.psi[i], ".", name.Z[i])
-    #nomiVxb <- unlist(mapply(forma.nomiVxb, length.psi, name.Z))
     nomiU   <- unlist(mapply(forma.nomiU, length.psi, nomiFINALI)) #invece di un ciclo #paste("U",1:length.psi[i], ".", name.Z[i])
     nomiVxb <- unlist(mapply(forma.nomiVxb, length.psi, nomiFINALI))
 
@@ -403,11 +431,12 @@ if(!is.null(nomiNO)) mfExt$formula<-update.formula(mfExt$formula,paste(".~.-", p
         return(objF)      
         }
     }
+    #browser()
     if(!gap){
       names.coef<-names(objF$coefficients)
       #questi codici funzionano e si basano sull'assunzioni che le U e le V siano ordinate..
       names(obj$coefficients)[match(c(paste("U",1:k, sep=""), paste("V",1:k, sep="")), names(coef(obj)))]<- nnomi  
-      objF$coefficients[names.coef]<-obj$coefficients[names.coef] #sostituisce gli 0 
+      objF$coefficients[names.coef]<-obj$coefficients[names.coef] #sostituisce tutti i coef (gli ultimi sono 0)  
       #objF$coefficients<-obj$coefficients
       #names(objF$coefficients)<-names.coef
       objF$fitted.values<-obj$fitted.values
@@ -454,9 +483,11 @@ if(!is.null(nomiNO)) mfExt$formula<-update.formula(mfExt$formula,paste(".~.-", p
     objF$id.warn <- id.warn
     objF$orig.call<-orig.call
     objF$indexU<-build.all.psi(psi.list, fixed.psi)
+    #browser()
     if(model)  objF$model <- mf #objF$mframe <- data.frame(as.list(KK))
-    if(n.boot>0) objF$seed<-employed.Random.seed
+    if(n.boot>0) objF$seed <- seed
     class(objF) <- c("segmented", class(obj0))
+    objF$psi[,"Initial"]<-NA
     list.obj[[length(list.obj) + 1]] <- objF
     class(list.obj) <- "segmented"
     if (last)
